// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 epauxcomb

   Authors:  Hassan Siddiqui - hsiddiqu@astro.estec.esa.nl
         
   Description: Combines the two tables of a PN ODF Auxiliary file
 
   Comments: 
 
  $Id: epauxcomb.cc,v 1.1 2000/03/31 08:49:31 hsiddiqu Exp $
*/

/* General includes */

/* SAS-specific includes */
#include "param.h"
#include "Dal.h"
#include "Tracer.h"

/* Local includes */
#include "epauxcomb.h"

void epauxcomb ()
{
  PROC;
  
  string setname = stringParameter("set");
  string outsetname = stringParameter("outset");

  dataSetServer->copy(setname, outsetname);

  DataSet* set = dataSetServer->open(outsetname, DataSet::Modify);

  Table * table = set->table("PNAUX1");
  Table * table2 = set->table("PNAUX2");
  
  /* Access NABOVE column in table 2, and create a new column of the same
     name in table 1
  */
  Column* nabove2Col              = table2->column("NABOVE"); 
  ColumnData* nabove2ColData      = nabove2Col->data();
  uint16* nabove2Data              = nabove2ColData->uint16Data();

  Column* new_naboveCol           = table->addColumn( nabove2Col->name(), 
						      nabove2Col->dataType(), 
						      nabove2Col->label() );
  ColumnData* new_naboveColData   = new_naboveCol->data();
  uint16* new_naboveData           = new_naboveColData->uint16Data();

  
  /* Access NDEFA column in table 2, and create a new column of the same
     name in table 1
  */
  Column* ndefa2Col               = table2->column("NDEFA"); 
  ColumnData* ndefa2ColData       = ndefa2Col->data();
  int16* ndefa2Data               = ndefa2ColData->int16Data();

  Column* new_ndefaCol            = table->addColumn( ndefa2Col->name(), 
						      ndefa2Col->dataType(), 
						      ndefa2Col->label() );
  ColumnData* new_ndefaColData    = new_ndefaCol->data();
  int16* new_ndefaData            = new_ndefaColData->int16Data();

  /* Access NEPDH column in table 2, and create a new column of the same
     name in table 1
  */
  Column* nepdh2Col               = table2->column("NEPDH"); 
  ColumnData* nepdh2ColData       = nepdh2Col->data();
  uint16* nepdh2Data               = nepdh2ColData->uint16Data();

  Column* new_nepdhCol            = table->addColumn( nepdh2Col->name(), 
						      nepdh2Col->dataType(), 
						      nepdh2Col->label() );
  ColumnData* new_nepdhColData    = new_nepdhCol->data();
  uint16* new_nepdhData            = new_nepdhColData->uint16Data();

  

  /* Access NDISCLIN column in table 2, and create a new column of the same
     name in table 1
  */
  Column* ndisclin2Col            = table2->column("NDISCLIN"); 
  ColumnData* ndisclin2ColData    = ndisclin2Col->data();
  int16* ndisclin2Data            = ndisclin2ColData->int16Data();

  Column* new_ndisclinCol         = table->addColumn( ndisclin2Col->name(), 
						      ndisclin2Col->dataType(), 
						      ndisclin2Col->label() );
  ColumnData* new_ndisclinColData = new_ndisclinCol->data();
  int16* new_ndisclinData         = new_ndisclinColData->int16Data();


  /* Access MCOMMODE column in table 2, and create a new column of the same
     name in table 1
  */
  Column* mcommode2Col            = table2->column("MCOMMODE"); 
  ColumnData* mcommode2ColData    = mcommode2Col->data();
  uint16* mcommode2Data            = mcommode2ColData->uint16Data();

  Column* new_mcommodeCol         = table->addColumn( mcommode2Col->name(), 
						      mcommode2Col->dataType(), 
						      mcommode2Col->label() );
  ColumnData* new_mcommodeColData = new_mcommodeCol->data();
  uint16* new_mcommodeData         = new_mcommodeColData->uint16Data();

  
  /* Populate the columns by relating 
   * the CYCLE and QUADRANT information in the first and 
   * second extensions in the original dataset. */

  Column* cycleCol              = table->column("CYCLE");
  ColumnData* cycleColData      = cycleCol->data();
  int32* cycleData              = cycleColData->int32Data();
  
  Column* quadrantCol           = table->column("QUADRANT");
  ColumnData * quadrantColData  = quadrantCol->data();
  int8*  quadrantData           = quadrantColData->int8Data();

  Column* cycle2Col             = table2->column("CYCLE");
  ColumnData * cycle2ColData    = cycle2Col->data();
  int32*  cycle2Data            = cycle2ColData->int32Data();

  Column* quadrant2Col          = table2->column("QUADRANT");
  ColumnData * quadrant2ColData = quadrant2Col->data();
  int8*  quadrant2Data          = quadrant2ColData->int8Data();
  
  unsigned table1rows = table->rows();

  /* Initialise the values along all new columns to zero */
  for (unsigned index = 0; index < table1rows; ++index)
    {
      new_naboveData[index]   = 0;
      new_ndefaData[index]    = 0;
      new_nepdhData[index]    = 0;
      new_ndisclinData[index] = 0;
      new_mcommodeData[index] = 0;
    }

  /* NOTE: Algorithm assumes that cycle and quadrant data in
     both tables are monotonically increasing/decreasing.
   */
  unsigned table2rows = table2->rows();
  unsigned index2 = 0;
  for (unsigned long index = 0; 
       index < table1rows && index2 < table2rows; 
       ++index)
    {
      while (true)
	   {
	     if ( (cycleData[index] == cycle2Data[index2]) &&
		  (quadrantData[index] == quadrant2Data[index2]) )
	       {
		 new_naboveData[index]   = nabove2Data[index2];
		 new_ndefaData[index]    = ndefa2Data[index2];
		 new_nepdhData[index]    = nepdh2Data[index2];
		 new_ndisclinData[index] = ndisclin2Data[index2];
		 new_mcommodeData[index] = mcommode2Data[index2];

		 errstr 
		   << " Table 1 index " << index
		   << " matches Table 2 index " << index2
		   << Sas::message(Msg::App, Msg::Verbose);
		 break;
	       }

	     index2++;

	     if (index2 >= table2rows)
	       {
		 errstr 
		   << " Found no matches for table 1 index " << index
		   << ". Processing considered complete."
		   << Sas::message(Msg::App, Msg::Verbose);
		 break;
	       }
	   }
    }

  /* Cleanup */
  delete nabove2ColData;
  delete new_naboveColData;
  delete ndefa2ColData;
  delete new_ndefaColData;
  delete nepdh2ColData;
  delete new_nepdhColData;
  delete ndisclin2ColData;
  delete new_ndisclinColData;
  delete mcommode2ColData;
  delete new_mcommodeColData;

  delete cycleColData;
  delete cycle2ColData;
  delete quadrantColData;
  delete quadrant2ColData;

  dataSetServer->close(set);


  RETURN;
    
}
